# -*- coding: utf-8 -*-

"""
genres.db
---

This module handles the genres database.
"""

import os


class Db:
    reference = None
    genres = None
    tags = None

    def __init__(self, data_path=None):
        if not data_path:
            data_path = "%s/data.txt" % os.path.dirname(
                os.path.abspath(__file__))

        data = self.load(data_path)
        self.parse(data)

    def load(self, data_path):
        with open(data_path, "r") as data_file:
            raw_data = data_file.read()

        data_file.close()
        return raw_data

    def parse(self, data):
        categories = data.split("\n\n")
        reference = {}
        genre_index = []
        tag_index = []

        for category in categories:
            entries = category.strip().split("\n")
            entry_category = entries[0].lower()
            for entry in entries:
                entry = entry.lower()
                if not entry:
                    continue

                # Comment, ignore
                if entry.startswith("#"):
                    continue

                # Handle genre
                if not entry.startswith("-"):
                    genre = entry

                    reference[genre] = entry_category
                    genre_index.append(genre)

                # Handle tag
                else:
                    tag = entry[1:]
                    tag = tag.strip()

                    reference[tag] = entry_category
                    tag_index.append(tag)

        self.reference = reference
        self.genres = genre_index
        self.tags = tag_index



# -*- coding: utf-8 -*-

"""
genres.finder
---

This module handles the genre finder.
"""

from collections import Counter
import re


class Finder():
    db = None
    unique_category = True

    def __init__(self, db, unique_category=True):
        self.db = db
        self.unique_category = unique_category

    def contains_entity(self, entity, text):
        try:
            entity = re.escape(entity)
            entity = entity.replace("\ ", "([^\w])?")
            found = re.search("(\ |-|\|/|\.|,|^)%s(\ |\-|\|/|\.|,|$)" % entity,
                text, re.I | re.M)
        except Exception, e:
            found = False

        return found

    def find(self, text):
        genres = []
        text = text.lower()

        category_counter = Counter()
        counter = Counter()

        for genre in self.db.genres:
            if self.contains_entity(genre, text):
                counter[genre] += 1
                category = self.db.reference[genre]
                category_counter[category] += 1

        for tag in self.db.tags:
            if self.contains_entity(tag, text):
                category = self.db.reference[tag]

                # Only increase main category count once when tag is found.
                if not counter[category]:
                    counter[category] += 1

                category_counter[category] += 1

        if len(category_counter) == 0:
            return genres

        main_category = category_counter.most_common(1)[0][0]

        # Convert counter to a flat list of genres, sorted by count
        sorted_genres = [ite for ite, it in counter.most_common()]

        for genre in sorted_genres:
            insert = True

            if self.unique_category:
                if not self.db.reference[genre] == main_category:
                    insert = False

            if insert:
                genres.append(genre)

        return genres



# -*- coding: utf-8 -*-

"""
genres
---

Genres is a library that analyzes text with musical context and guesses genres.

    >>> import genres
    >>> r = genres.find("Pink Floyd is a great rock band)
    >>> r
    >>> ["Rock]

:copyright: (c) 2014 by Martin Sandström
:license: MIT, see LICENSE for more details.
"""

__title__ = "genres"
__version__ = "1.0"
__build__ = 1
__author__ = "Martin Sandström"
__license__ = "MIT"
__copyright__ = "Copyright 2014 Martin Sandström"


from .api import (
    find
)



# -*- coding: utf-8 -*-

"""
genres.api
---

This module implements the Genres API.
"""

import logging
from . import finder, db

_db = None


def find(text):
    """
    Tries to determine genres for the text.
    """

    finder_obj = _create_finder()
    return finder_obj.find(text)


def _get_database():
    """
    Check if database has been created, otherwise construct it.
    """

    global _db
    if not _db:
        db_obj = db.Db()
        _db = db_obj

    return _db


def _create_finder():
    """
    Create finder object based on db.
    """

    db_obj = _get_database()
    return finder.Finder(db_obj)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys

import genres

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

if sys.argv[-1] == "publish":
    os.system("python setup.py sdist upload")
    sys.exit()

packages = [
    "genres"
]

with open('README.md') as f:
    readme = f.read()

requires = []

long_description = """
Genres is a library that analyzes text with musical context and guesses genres.

---

%s

""" % (readme)


setup(
    name="genres",
    version=genres.__version__,
    description="Guesses genre for text with musical context",
    long_description=long_description,
    author="Martin Sandström",
    author_email="martin@marteinn.se",
    url="https://github.com/marteinn/genres",
    packages=packages,
    package_data={"": ["LICENSE", "reviews.txt"], "genres": ["*.txt"]},
    package_dir={"genres": "genres"},
    include_package_data=True,
    install_requires=requires,
    license="Apache 2.0",
    zip_safe=False,
    classifiers=(
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Natural Language :: English",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7"
    ),
    extras_require={
    },
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests"""

import re
import os
import unittest
from genres import find


class GenreTestCase(unittest.TestCase):
    def test_single_match(self):
        result = find("This is a good example of techno. Hello techno.")
        assert "techno" in result

    def test_multiple_match(self):
        result = find("This is Dance Hall. Sounds like a mix of Merengue")
        assert "dance hall" in result

    def multiple_and_invalid(self):
        result = find("KingstonDance Hall and Reggae")
        assert "dance hall" not in result

    def test_conflicting_genres(self):
        result = find("Dance Hall and Reggae-Techno")
        assert "techno" not in result

    def test_not_existing_genre(self):
        result = find("Lorem ipsum dolum test")
        assert len(result) == 0

    def test_highest_occurence_return_genre(self):
        result = find("ska, Techno, Ska")
        assert "ska" in result

    def test_comment_data(self):
        result = find("#comment")
        assert "#comment" not in result

    def test_tag_match(self):
        result = find("In this text we have a Country Test Tag")
        assert "country" in result

    def test_escaped_genre(self):
        result = find("For over a decade, Los Angeles singer/songwriter \
            Jhené Aiko has skirted the periphery of R&B stardom")
        assert "r&b" in result

    def test_between_words(self):
        result = find("Hip-hop is a genre")
        assert "hip hop" in result


class TestTagCase(unittest.TestCase):
    def test_genre_tag_occurence(self):
        result = find("Ska jazz. Ska jazz. The best jazz player is Miles Davis. \
        Miles Davis. Miles Davis")
        assert "ska jazz" == result[0]

    def test_tag_vs_genre_check(self):
        result = find("Miles Davis. Miles Davis. Miles Davis. Techno. Techno.")
        assert "jazz" in result
        assert len(result) == 1


class TestArticleContentCase(unittest.TestCase):
    reviews = []

    def setUp(self):
        data_path = "%s/reviews.txt" % os.path.dirname(
                os.path.abspath(__file__))

        with open(data_path, "r") as data_file:
            raw_data = data_file.read()

        data_file.close()
        raw_data = raw_data.strip()

        reviews = raw_data.split("\n")
        for review in reviews:
            review = review.strip()

            if len(review) == 0:
                continue

            if review[:1].startswith("#"):
                continue

            review_struct = review.split(";")
            category = review_struct[0].split(",")
            content = "".join(review_struct[1:])

            self.reviews.append([category, content])

    def tearDown(self):
        pass

    def test_reviews(self):
        for review in self.reviews:
            result = find(review[1])
            found = all((w in result for w in review[0]))
            if not found:
                raise AssertionError("%s Genres %s not found in %s",
                    review[1][:20], review[0], result)

if __name__ == "__main__":
    unittest.main()
